diff --git a/module/ice-9/boot-9.scm b/module/ice-9/boot-9.scm
index 872594b..f1ff35e 100644
--- a/module/ice-9/boot-9.scm
+++ b/module/ice-9/boot-9.scm
@@ -2703,10 +2703,15 @@ VALUE."
   (let ((accessor (record-accessor module-type 'name)))
     (lambda (mod)
       (or (accessor mod)
-          (let ((name (list (gensym))))
-            ;; Name MOD and bind it in the module root so that it's visible to
-            ;; `resolve-module'. This is important as `psyntax' stores module
-            ;; names and relies on being able to `resolve-module' them.
+          (let ((name `(%ephemeral ,(gensym))))
+            ;; Give MOD a name so that it's visible to `resolve-module'.
+            ;; This is important as `psyntax' stores module names and
+            ;; relies on being able to `resolve-module' them.
+            ;;
+            ;; We make MOD a submodule of the special module
+            ;; (%ephemeral), which uses weak-key hash tables, so that
+            ;; the module can be reclaimed when the gensym in its name
+            ;; is no longer referenced.
             (set-module-name! mod name)
             (nested-define-module! (resolve-module '() #f) name mod)
             (accessor mod))))))
@@ -2795,6 +2800,10 @@ VALUE."
               (and ensure
                    (make-modules-in root name)))))))))
 
+;; Now create the special (%ephemeral) module with weak-key hash tables.
+(let ((m (resolve-module '(%ephemeral) #f)))
+  (set-module-obarray!    m (make-weak-key-hash-table))
+  (set-module-submodules! m (make-weak-key-hash-table)))
 
 (define (try-load-module name version)
   (try-module-autoload name version))
